"""This file provides a simple segmenter that splits texts based on regex. 
The default segmenter takes xAIF, segments the texts in each L-node, 
introduces new L-node entries for each of the new segments, and deletes the old L-node entries.
"""

import re
from flask import json
import logging
import spacy

# Load the pre-trained spaCy model for English
nlp = spacy.load("en_core_web_sm")
from src.data import Data
from xaif_eval import xaif
from src.templates import SegmenterOutput
logging.basicConfig(datefmt='%H:%M:%S',
                    level=logging.DEBUG)

class Segmenter():
	def __init__(self,file_obj):
		self.file_obj = file_obj
		self.f_name = file_obj.filename
		self.file_obj.save(self.f_name)
		file = open(self.f_name,'r')

	def get_segments(self, input_text):
		"""Split input text into sentences using spaCy's sentence segmentation."""
		doc = nlp(input_text)  # Process the input text using spaCy
		return [sent.text.strip() for sent in doc.sents]  # Return sentences, stripping any extra spaces

	def is_valid_json(self):
		''' check if the file is valid json
		'''

		try:
			json.loads(open(self.f_name).read())
		except ValueError as e:			
			return False

		return True
	def is_valid_json_aif(self,aif_nodes):
		if 'nodes' in aif_nodes and 'locutions' in aif_nodes and 'edges' in aif_nodes:
			return True
		return False
		

	def get_aif(self, format='xAIF'):

		with open(self.f_name) as file:
			data = file.read()
			x_aif = json.loads(data)
			if format == "xAIF":
				return x_aif
			else:
				aif = x_aif.get('AIF')
				return json.dumps(aif)


	def segmenter_default(self,):
		"""The default segmenter takes xAIF, segments the texts in each L-nodes,
		introduce new L-node entries for each of the new segements and delete the old L-node entries
		"""
		xAIF_input = self.get_aif()
		logging.info(f"xAIF data:  {xAIF_input}, {self.file_obj}")  
		xaif_obj = xaif.AIF(xAIF_input)
		is_json_file = self.is_valid_json()
		if is_json_file:				
			json_dict = xaif_obj.aif
			if self.is_valid_json_aif(json_dict):
				nodes = json_dict['nodes']		
				for nodes_entry in nodes:
					node_id = nodes_entry['nodeID']
					node_text = nodes_entry['text']
					type = nodes_entry['type']
					if type == "L":
						segments = self.get_segments(node_text)
						segments = [seg.strip() for seg in segments if len(seg.strip()) > 1]
						if len(segments) > 1:
							for segment in segments:								
								if segment != "":	
									xaif_obj.add_component("segment", node_id, segment)										


				return xaif_obj.xaif
			else:
				return("Invalid json-aif")
		else:
			return("Invalid input")
	

'''
{"AIF":{"edges":[{"edgeID":"1","formEdgeID":"None","fromID":"247603","toID":"247602"},{"edgeID":"2","formEdgeID":"None","fromID":"247604","toID":"247603"}],"locutions":[],
"nodes":[{"nodeID":"1","text":"THANK YOU","timestamp":"2016-10-31 17:17:34","type":"I"},{"nodeID":"2","text":"COOPER : THANK YOU","timestamp":"2016-11-10 18:34:23","type":"L"},
{"nodeID":"3","text":"You are well come","timestamp":"2016-10-31 17:17:34","type":"I"},{"nodeID":"4","text":"Bob : You are well come","timestamp":"2016-11-10 18:34:23","type":"L"},
{"nodeID":"5","text":"does or doesn't Jeane Freeman think the SNP is divided with what is going on","timestamp":"","type":"I"},
{"nodeID":"6","text":"the SNP is a big party","timestamp":"","type":"I"},{"nodeID":"7","text":"would or wouldn't Jeane Freeman describe the SNP as united","timestamp":"","type":"I"},
{"nodeID":"8","text":"the SNP has disagreements","timestamp":"","type":"I"},{"nodeID":"9","text":"the SNP has disagreements","timestamp":"","type":"I"},
{"nodeID":"10","text":"Michael Forsyth belongs to a party that has disagreements","timestamp":"","type":"I"},
{"nodeID":"11","text":"one disagreement of Michael Forsyth's party is currently about their Scottish leader","timestamp":"","type":"I"},
{"nodeID":"12","text":"Iain Murray has had disagreements with his party","timestamp":"","type":"I"},
{"nodeID":"13","text":"it's not uncommon for there to be disagreements between party members","timestamp":"","type":"I"},
{"nodeID":"14","text":"disagreements between party members are entirely to be expected","timestamp":"","type":"I"},
{"nodeID":"15","text":"what isn't acceptable is any disagreements are conducted that is disrespectful of other points of view","timestamp":"","type":"I"},
{"nodeID":"16","text":"Jeanne Freeman wants to be in a political party and a country where different viewpoints. and different arguments, Donald Dyer famously said, are conducted with respect and without abuse","timestamp":"","type":"I"},{"nodeID":"17","text":"who does or doesn't Jeanne Freeman think is being disrespectful then","timestamp":"","type":"I"},{"nodeID":"18","text":"people feel, when they have been voicing opinions on different matters, that they have been not listened to","timestamp":"","type":"I"},
{"nodeID":"19","text":"people feel that they have been treated disrespectfully on all sides of the different arguments and disputes going on","timestamp":"","type":"I"}],"participants":[]}}




{"aif":{"edges":[{"edgeID":"1","formEdgeID":"None","fromID":"1","toID":"20"},{"edgeID":"2","formEdgeID":"None","fromID":"20","toID":"3"}],"locutions":[],
"nodes":[{"nodeID":"1","text":"THANK YOU","timestamp":"2016-10-31 17:17:34","type":"I"},
{"nodeID":"2","text":"COOPER : THANK YOU","timestamp":"2016-11-10 18:34:23","type":"L"},
{"nodeID":"3","text":"You are well come","timestamp":"2016-10-31 17:17:34","type":"I"},
{"nodeID":"4","text":"Bob : You are well come","timestamp":"2016-11-10 18:34:23","type":"L"},
{"nodeID":"5","text":"does or doesn't Jeane Freeman think the SNP is divided with what is going on","timestamp":"","type":"I"},
{"nodeID":"6","text":"the SNP is a big party","timestamp":"","type":"I"},{"nodeID":"20","text":"Default Inference","timestamp":"","type":"RA"},
{"nodeID":"7","text":"would or wouldn't Jeane Freeman describe the SNP as united","timestamp":"","type":"I"},
{"nodeID":"8","text":"the SNP has disagreements","timestamp":"","type":"I"},{"nodeID":"9","text":"the SNP has disagreements","timestamp":"","type":"I"},
{"nodeID":"10","text":"Michael Forsyth belongs to a party that has disagreements","timestamp":"","type":"I"},
{"nodeID":"11","text":"one disagreement of Michael Forsyth's party is currently about their Scottish leader","timestamp":"","type":"I"},
{"nodeID":"12","text":"Iain Murray has had disagreements with his party","timestamp":"","type":"I"},
{"nodeID":"13","text":"it's not uncommon for there to be disagreements between party members","timestamp":"","type":"I"},
{"nodeID":"14","text":"disagreements between party members are entirely to be expected","timestamp":"","type":"I"},
{"nodeID":"15","text":"what isn't acceptable is any disagreements are conducted that is disrespectful of other points of view","timestamp":"","type":"I"},
{"nodeID":"16","text":"Jeanne Freeman wants to be in a political party and a country where different viewpoints and different arguments, Donald Dyer famously said, are conducted with respect and without abuse","timestamp":"","type":"I"},
{"nodeID":"17","text":"who does or doesn't Jeanne Freeman think is being disrespectful then","timestamp":"","type":"I"},
{"nodeID":"18","text":"people feel, when they have been voicing opinions on different matters, that they have been not listened to","timestamp":"","type":"I"},
{"nodeID":"19","text":"people feel that they have been treated disrespectfully. on all sides of the different arguments and disputes going on","timestamp":"","type":"I"}],"participants":[]}}
'''




  



import requests
import os
import tempfile
import json
import logging
from collections import deque


class PipelineBuilder:
    def __init__(self,Deployer):
        self.deployer = Deployer

    def _build_dependency_graph(self, pipeline_graph):
        """Build the dependency graph from the pipeline specification."""
        # Initialize an empty list for each module to hold dependencies
        module_dependencies = {module: [] for module in self.deployer.modules.keys()}
        for dependency in pipeline_graph:
            # Check if the dependency is a tuple where the second element is a list of dependencies
            if isinstance(dependency[1], list):
                for dep in dependency[1]:
                    module_dependencies[dependency[0]].append(dep)
            else:
                module_dependencies[dependency[0]].append(dependency[1])

        return module_dependencies
    def resolve_dependencies(self, pipeline_graph):
        # Create a dictionary to track dependencies
        module_dependencies = self._build_dependency_graph(pipeline_graph)
        #print("the whole row module dependencies", module_dependencies)
        # List to track the visited modules
        visited = set()

        def visit(module, stack):
            if module in stack:
                raise ValueError(f"Circular dependency detected: {' -> '.join(stack + [module])}")
            if module in visited:
                return
            visited.add(module)
            stack.append(module)
            for dep in module_dependencies.get(module, []):
                visit(dep, stack)
            stack.pop()

        # Check each module
        for module in module_dependencies:
            visit(module, [])
        return module_dependencies



    def new_pipeline(self, pipeline_graph_with_tags):
        """
        Install and set up only the modules specified in the pipeline, with tag tracking.
        :param pipeline_graph: List of module names and edges (dependencies) in the pipeline.
        :param tag_to_module: Dictionary mapping tags to module names.
        """

        # Convert tag-based pipeline to actual module-based pipeline
        tag_to_module = self.deployer.modules_to_load
        pipeline_graph = [(tag_to_module[src], tag_to_module[dst]) for src, dst in pipeline_graph_with_tags]       
        execution_pipeline = {}  # Graph representation
        ordered_modules = self.resolve_dependencies(pipeline_graph)


        # To track modules that have been added to the execution pipeline
        added_to_pipeline = set()

        def add_module(module_name, tag=None):
            """
            Process and add a module to the pipeline graph, maintaining dependencies.
            :param module_name: The name of the module to add.
            :param tag: The tag associated with this module (for tracking).
            """
            if module_name in added_to_pipeline:
                return  # Skip if the module is already processed

            module = self.deployer.modules.get(module_name)
            if module:
                module_url = module['url']
                module_type = module['type']
                route = module['route']
                destination = os.path.join(self.deployer.modules_dir, module_name)
                try:
                    if module_type == "repo":
                        print(f"Processing module '{module_name}' of type '{module_type}'...")
                        # Check if the module is already cloned
                        if not os.path.exists(destination):
                            self.deployer.clone_repository(module_url, destination)
                        else:
                            print(f"Module '{module_name}' already cloned. Skipping clone.")
                        # Check if the container is already running
                        if not self.deployer.is_container_running(module_name):
                            self.deployer.ensure_docker_setup(destination)
                            self.deployer.build_docker_image(destination)
                            self.deployer.start_docker_container(destination)
                        else:
                            print(f"Container for module '{module_name}' is already running. Skipping Docker setup.")
                        # Get the service port from the compose file
                        port = self.deployer.get_service_port_from_compose(destination, service_name=module_name)
                        execution_pipeline[module_name] = {
                            "name": module_name,
                            "url": f"http://localhost:{port}/{route}",
                            "dependencies": []  # Store dependencies in the graph
                        }
                    elif module_type == "ws":
                        print(f"Module '{module_name}' is of type 'ws'. No local setup needed.")
                        execution_pipeline[module_name] = {
                            "name": module_name,
                            "url": module_url,
                            "dependencies": []  # Store dependencies in the graph
                        }
                    else:
                        print(f"Unsupported module type '{module_type}' for module '{module_name}'. Skipping...")
                except Exception as e:
                    print(f"Failed to set up module '{module_name}': {str(e)}")

                # Mark module as added to the pipeline
                added_to_pipeline.add(module_name)

                # If a tag was passed, associate it in the execution pipeline
                if tag:
                    execution_pipeline[module_name]["tag"] = tag
        # Iterate over each module and its dependencies in the resolved graph
        for module_name, dependencies in ordered_modules.items():
            print(f"Processing module '{module_name}' and its dependencies...")

            # First, process the module itself (if it hasn't been processed already)
            # We need to ensure that we fetch the correct tag for this particular module
            tags_for_module = [tag for tag, mod_name in tag_to_module.items() if mod_name == module_name]
            
            # If there are multiple tags for the same module, we associate all of them in the execution pipeline
            for tag in tags_for_module:
                add_module(module_name, tag)

            # Then process each of its dependencies
            for dep in dependencies:
                tags_for_module = [tag for tag, mod_name in tag_to_module.items() if mod_name == dep]
                for tag in tags_for_module:
                    add_module(dep,tag)
                # Add dependency relationship in the graph
                execution_pipeline[module_name]["dependencies"].append(dep)



        return self._create_pipeline_executor(execution_pipeline, tag_to_module)


    def _create_pipeline_executor(self, execution_graph, tag_to_module):
        """
        Creates a pipeline execution function based on a dependency-aware execution order, with tag tracking.
        :param execution_graph: Dictionary representing the execution graph.
        :param tag_to_module: Dictionary mapping tags to module names.
        :return: Callable pipeline executor.
        """

        reverse_graph = {module: [] for module in execution_graph}

        # Reverse dependencies graph: Find which modules depend on each module
        for prerequisite, dependents in execution_graph.items():
            for dependent in dependents["dependencies"]:
                reverse_graph[dependent].append(prerequisite)

        print("Reverse dependency graph:", reverse_graph)

        executable_modules = [mod for mod, deps in reverse_graph.items() if not deps]
        module_outputs = {}
        print("executable_modules==========", executable_modules)
        def pipeline_executor(input_file):
            """Executes the pipeline following the dependency graph."""
            processed_modules = set()
            execution_queue = deque(executable_modules)
            module_outputs.update({mod: input_file for mod in executable_modules})
            previous_module = None

            while execution_queue:
                module_name = execution_queue.popleft()  # Process one module at a time
                module = execution_graph[module_name]

          

                # Get the tag for the current module
                module_tag = module.get("tag", "Unknown")

                # Identify the module providing the input for this module
                input_sources = [dep for dep in reverse_graph[module_name] if dep in module_outputs]
                input_modules = [tag_to_module.get(dep, dep) for dep in input_sources]

                # Print execution details, including the module that provides the input
                input_details = " and ".join(input_modules) if input_modules else "Original input"
                print(f"Executing module '{module_name}' (tag: {module_tag}), input provided by {module['url']}...")

                self.deployer.wait_for_service(module['url'])

                input_files = [module_outputs[dep] for dep in reverse_graph[module_name] if dep in module_outputs]
                if not input_files:
                    input_files = [input_file]  # Default to original input if no dependencies

                current_file_path = input_files[0]

                with open(current_file_path, "rb") as file_data:
                    file = {'file': (os.path.basename(current_file_path), file_data)}

                    try:
                        response = requests.post(module["url"], files=file)

                        if response.status_code == 200:
                            json_response = response.json()
                            #print(f"Module '{module_name}' output: {json_response}")

                            with tempfile.NamedTemporaryFile(delete=False, mode='w', encoding='utf-8', suffix='.json') as temp_file:
                                json.dump(json_response, temp_file)
                                temp_file_path = temp_file.name
                                module_outputs[module_name] = temp_file_path
                            print(f"Successfully processed '{module_name}'. Output saved at {temp_file_path}")

                            processed_modules.add(module_name)

                            # Add dependent modules to the execution queue
                            for dependent in execution_graph[module_name]["dependencies"]:
                                if all(dep in processed_modules for dep in reverse_graph[dependent]):
                                    execution_queue.append(dependent)

                        else:
                            print(f"Failed at '{module_name}' with status {response.status_code}: {response.text}")
                            break

                    except Exception as e:
                        logging.error(f"Error processing module '{module_name}': {str(e)}")
                        break
                previous_module = module_tag

            return module_outputs.get(list(execution_graph.keys())[-1], input_file)

        return pipeline_executor


    







from src.bert_te import BertArgumentStructure 
from src.utility import handle_errors 
from src.models import Model

from flask import Flask, request,render_template_string
from prometheus_flask_exporter import PrometheusMetrics
import logging
import json
import markdown2




logging.basicConfig(datefmt='%H:%M:%S', level=logging.DEBUG)

def load_config(file_path):
        """Load the contents of the config.json file to get the model files."""
        with open(file_path, 'r') as config_file:
            config_data = json.load(config_file)
            return config_data.get('model_path')

app = Flask(__name__)
metrics = PrometheusMetrics(app)

config_file_path = "config/config.json"
model_path = load_config(config_file_path)
model = Model(model_path)

@metrics.summary('requests_by_status', 'Request latencies by status',
                 labels={'status': lambda r: r.status_code})
@metrics.histogram('requests_by_status_and_path', 'Request latencies by status and path',
                   labels={'status': lambda r: r.status_code, 'path': lambda: request.path})
@handle_errors
@app.route('/bert-te', methods = ['GET', 'POST'])
#
def bertte():
	if request.method == 'POST':
		file_obj = request.files['file']
		result = BertArgumentStructure(file_obj,model).get_argument_structure()

		return result
	
	if request.method == 'GET':
		# Read the markdown file
		with open('README.md', 'r') as file:
			md_content = file.read()

		# Convert to HTML
		html_content = markdown2.markdown(md_content)

		# Add CSS link
		css_link = '<link rel="stylesheet" href="https://example.com/path/to/your/styles.css">'
		html_with_css = f"<html><head>{css_link}</head><body>{html_content}</body></html>"

		# Render the HTML content as a template
		return render_template_string(html_with_css)
		

	
if __name__ == "__main__":
    app.run(host="0.0.0.0", port=int("5002"), debug=False)	  



import json
from typing import Dict, List


    
class AIF:
    def __init__(self, ):
        pass
    def is_valid_json_aif(sel,aif_nodes):
        if 'nodes' in aif_nodes and 'locutions' in aif_nodes and 'edges' in aif_nodes:
            return True
        return False
    def is_json_aif_dialog(self, aif_nodes: list) -> bool:
        ''' check if json_aif is dialog
        '''

        for nodes_entry in aif_nodes:					
            if nodes_entry['type'] == "L":
                return True
        return False
    


    def get_next_max_id(self, nodes, n_type):
        """
       Takes a list of nodes (edges) and returns the maximum node/edge ID.
        Arguments:
        - nodes/edges (List[Dict]): a list of nodes/edges, where each node is a dictionary containing a node/edge ID
        Returns:
        - (int): the maximum node/edge ID in the list of nodes
        """

        max_id, lef_n_id, right_n_id = 0, 0, ""
        if isinstance(nodes[0][n_type],str): # check if the node id is a text or integer
            if "_" in nodes[0][n_type]:
                for node in nodes:
                    temp_id = node[n_type]
                    if "_" in temp_id:
                        nodeid_parsed = temp_id.split("_") # text node id can involve the character "_"
                        lef_n_id, right_n_id = int(nodeid_parsed[0]), nodeid_parsed[1]
                        if lef_n_id > max_id:
                            max_id = lef_n_id
                return str(int(max_id)+1)+"_"+str(right_n_id)
            else:
                for node in nodes:
                    temp_id = int(node[n_type])     
                    if temp_id > max_id:
                        max_id = temp_id   
                return str(max_id+1)

        elif isinstance(nodes[0][n_type],int):	
            for node in nodes:
                temp_id = node[n_type]     
                if temp_id > max_id:
                    max_id = temp_id   
            return max_id+1
        


    def get_speaker(self, 
        node_id: int, 
        locutions: List[Dict[str, int]], 
        participants: List[Dict[str, str]]
        ) -> str:
        """
        Takes a node ID, a list of locutions, and a list of participants, and returns the name of the participant who spoke the locution with the given node ID, or "None" 
        if the node ID is not found.

        Arguments:
        - node_id (int): the node ID to search for
        - locutions (List[Dict]): a list of locutions, where each locution is a dictionary containing a node ID and a person ID
        - participants (List[Dict]): a list of participants, where each participant is a dictionary containing a participant ID, a first name, and a last name

        Returns:
        - (str): the name of the participant who spoke the locution with the given node ID, or "None" if the node ID is not found
        """

        nodeID_speaker = {}
        # Loop through each locution and extract the person ID and node ID
        for locution in locutions:
            personID = locution['personID']
            nodeID = locution['nodeID']
            
            # Loop through each participant and check if their participant ID matches the person ID from the locution
            for participant in participants:
                if participant["participantID"] == personID:
                    # If there is a match, add the participant's name to the nodeID_speaker dictionary with the node ID as the key
                    firstname = participant["firstname"]
                    surname = participant["surname"]
                    nodeID_speaker[nodeID] = (firstname+" "+surname,personID)
                    
        # Check if the given node ID is in the nodeID_speaker dictionary and return the corresponding speaker name, or "None" if the node ID is not found
        if node_id in nodeID_speaker:
            return nodeID_speaker[node_id]
        else:
            return ("None None","None")

    def create_turn_entry(
        self,
        nodes, 
        node_id,
        person_id, 
        text_with_span,
        propositions,
        locutions,
        participants,
        dialogue 
        ):
        if dialogue:
            for first_and_last_names, proposition in propositions:
                first_last_names = first_and_last_names.split()
                first_names, last_names = "None", "None"
                if len(first_last_names) > 1:
                    first_names,last_names = first_last_names[0],first_last_names[1]
                else:
                    first_names, last_names = first_last_names[0],"None"
                text = proposition.replace("\n","")
                nodes.append({'text': text, 'type':'L','nodeID': node_id})
                locutions.append({'personID': person_id, 'nodeID': node_id})
                # Check if the entry already exists based on first name and surname
                if not any(participant['firstname'] == first_names and participant['surname'] == last_names for participant in participants):
                    participants.append({
                        "participantID": person_id,
                        "firstname": first_names,
                        "surname": last_names
                    })
                text_with_span = text_with_span+" "+first_names+" "+last_names+" "+"<span class=\"highlighted\" id=\""+str(node_id)+"\">"+text+"</span>.<br><br>"
                node_id = node_id + 1 
                person_id = person_id + 1


        else:
            text = propositions.replace("\n","")
            speaker = "Default Speaker"
            nodes.append({'text': text, 'type':'L','nodeID': node_id})	
            locutions.append({'personID': 1, 'nodeID': node_id})
            if not any(participant['firstname'] == "Default" and participant['surname'] == "Speaker" for participant in participants):
                participants.append(
                        {
                        "participantID": 1,                                
                        "firstname": "Default",                                
                        "surname": "Speaker"
                        }
                    )	
            text_with_span=text_with_span+" "+speaker+" "+"<span class=\"highlighted\" id=\""+str(node_id)+"\">"+text+"</span>.<br><br>"
            node_id = node_id + 1
        return (
            nodes, 
            locutions,
            participants, 
            text_with_span, 
            node_id,
            person_id
            )
    
    def get_i_node_ya_nodes_for_l_node(self, edges, n_id):
        """traverse through edges and returns YA node_ID and I node_ID, given L node_ID"""
        for entry in edges:
            if n_id == entry['fromID']:
                ya_node_id = entry['toID']
                for entry2 in edges:
                    if ya_node_id == entry2['fromID']:
                        inode_id = entry2['toID']
                        return(inode_id, ya_node_id)
        return None, None
    

    def remove_entries(self, l_node_id, nodes, edges, locutions):
        """
        Removes entries associated with a specific node ID from a JSON dictionary.

        Arguments:
        - node_id (int): the node ID to remove from the JSON dictionary
        - json_dict (Dict): the JSON dictionary to edit

        Returns:
        - (Dict): the edited JSON dictionary with entries associated with the specified node ID removed
        """
        # Remove nodes with the specified node ID
        in_id, yn_id = self.get_i_node_ya_nodes_for_l_node(edges, l_node_id)
        edited_nodes = [node for node in nodes if node.get('nodeID') != l_node_id]
        edited_nodes = [node for node in edited_nodes if node.get('nodeID') != in_id]

        # Remove locutions with the specified node ID
        edited_locutions = [node for node in locutions if node.get('nodeID') != l_node_id]

        # Remove edges with the specified node ID
        edited_edges = [node for node in edges if not (node.get('fromID') == l_node_id or node.get('toID') == l_node_id)]
        edited_edges = [node for node in edited_edges if not (node.get('fromID') == in_id or node.get('toID') == in_id)]
        edited_nodes = [node for node in edited_nodes if node.get('nodeID') != yn_id]
        # Return the edited JSON dictionary
        return edited_nodes, edited_edges, edited_locutions
    

    def get_xAIF_arrays(self, aif_section, xaif_elements) -> tuple:
        """
        Extracts values associated with specified keys from the given AIF section dictionary.

        Args:
            aif_section (dict): A dictionary containing AIF section information.
            xaif_elements (List): A list of keys for which values need to be extracted from the AIF section.

        Returns:
            tuple: A tuple containing values associated with the specified keys from the AIF section.
        """
        # Extract values associated with specified keys from the AIF section dictionary
        # If a key is not present in the dictionary, returns an empty list as the default value
        return tuple(aif_section.get(element) for element in xaif_elements)


	









import json
from src.data import Data, AIF
from xaif_eval import xaif
from itertools import combinations
class BertArgumentStructure:
    def __init__(self,file_obj,model):
        self.file_obj = file_obj


        self.model = model

        self.file_obj = file_obj
        self.f_name = file_obj.filename
        self.file_obj.save(self.f_name)
        file = open(self.f_name,'r')

        xAIF_input = self.get_aif()  
        self.aif_obj = xaif.AIF(xAIF_input)

    def is_valid_json(self):
        ''' check if the file is valid json
		'''

        try:
            json.loads(open(self.f_name).read())
        except ValueError as e:			
            return False

        return True
    def is_valid_json_aif(sel,aif_nodes):
        if 'nodes' in aif_nodes and 'locutions' in aif_nodes and 'edges' in aif_nodes:
            return True
        return False

    def get_aif(self, format='xAIF'):
        if self.is_valid_json():
            with open(self.f_name) as file:
                data = file.read()
                x_aif = json.loads(data)
                if format == "xAIF":
                    return x_aif
                else:
                    aif = x_aif.get('AIF')
                    return json.dumps(aif)
        else:
            return "Invalid json"



    def get_argument_structure(self):
        """Retrieve the argument structure from the input data."""
        data = self.get_aif()
        if not data:
            return "Invalid input"
        
        x_aif = self.aif_obj.xaif
        aif =  self.aif_obj.aif
        if not self.is_valid_aif(aif):
            return "Invalid json-aif"

        propositions_id_pairs = self.get_propositions_id_pairs(aif)
        self.update_node_edge_with_relations(propositions_id_pairs)

        return x_aif


    def is_valid_aif(self, aif):
        """Check if the AIF data is valid."""
        return 'nodes' in aif and 'edges' in aif

    def get_propositions_id_pairs(self, aif):
        """Extract proposition ID pairs from the AIF data."""
        propositions_id_pairs = {}
        for node in aif.get('nodes', []):
            if node.get('type') == "I":
                proposition = node.get('text', '').strip()
                if proposition:
                    node_id = node.get('nodeID')
                    propositions_id_pairs[node_id] = proposition
        return propositions_id_pairs
    
    def update_node_edge_with_relations_(self, propositions_id_pairs):
        """
        Update the nodes and edges in the AIF structure to reflect the new relations between propositions.
        """
        checked_pairs = set()
        for prop1_node_id, prop1 in propositions_id_pairs.items():
            for prop2_node_id, prop2 in propositions_id_pairs.items():
                if prop1_node_id != prop2_node_id:
                    pair1 = (prop1_node_id, prop2_node_id)
                    pair2 = (prop2_node_id, prop1_node_id)
                    if pair1 not in checked_pairs and pair2 not in checked_pairs:
                        checked_pairs.add(pair1)
                        checked_pairs.add(pair2)
                        predictions = self.model.predict((prop1, prop2))
                        for prediction in predictions:
                            if prediction in ['RA','MA','CA']:
                                self.aif_obj.add_component("argument_relation", prediction, prop1_node_id, prop2_node_id)
                            



    def update_node_edge_with_relations(self, propositions_id_pairs, batch_size=4):
        """
        Update the nodes and edges in the AIF structure to reflect the new relations between propositions.
        """
        pairs_to_predict = []
        pair_ids = []

        # Use combinations to create pairs without redundant checking
        for (prop1_node_id, prop1), (prop2_node_id, prop2) in combinations(propositions_id_pairs.items(), 2):
            pairs_to_predict.append(prop1+ "" + prop2)
            pair_ids.append((prop1_node_id, prop2_node_id))

        # Process pairs in batches
        for i in range(0, len(pairs_to_predict), batch_size):
            batch_pairs = pairs_to_predict[i:i+batch_size]
            batch_pair_ids = pair_ids[i:i+batch_size]
            
            # Assuming `self.model.predict` can handle batches of inputs
            predictions = self.model.predict(batch_pairs)
            
            for (prop1_node_id, prop2_node_id), prediction in zip(batch_pair_ids, predictions):
                if prediction in ['RA', 'MA', 'CA']:
                    self.aif_obj.add_component("argument_relation", prediction, prop1_node_id, prop2_node_id)



import traceback
from flask import jsonify
# Decorator for error handling
def handle_errors(func):
    def wrapper(*args, **kwargs):
        try:
            return func(*args, **kwargs)
        except KeyError:
            return jsonify({'error': 'File not found in request'}), 400
        except Exception as e:
            traceback.print_exc()
            return jsonify({'error': str(e)}), 500
    return wrapper


from flask import Flask, request
from prometheus_flask_exporter import PrometheusMetrics
from flask_cors import CORS

from src.turninator import Turninator 
from src.util import handle_errors
import logging
#logging configuration
logging.basicConfig(datefmt='%H:%M:%S',
                    level=logging.DEBUG)


app = Flask(__name__)
CORS(app, resources={r"/*": {"origins": "https://arg-tech.github.io"}})

metrics = PrometheusMetrics(app)
	
@app.route('/turninator-01', methods = ['GET', 'POST'])
@metrics.summary('requests_by_status', 'Request latencies by status',
                 labels={'status': lambda r: r.status_code})
@metrics.histogram('requests_by_status_and_path', 'Request latencies by status and path',
                   labels={'status': lambda r: r.status_code, 'path': lambda: request.path})
@handle_errors  
def turninator_defult():
	if request.method == 'POST':
		file_obj = request.files.get('file')	
		turninator = Turninator(file_obj)
		result=turninator.turninator_default()
		return result	

	if request.method == 'GET':
		info = """Turninator is an AMF component that parses arguments into dialog turns.  
		This is the default implementation of a turninator that parses arguments using simple regular expressions. 
		It expects dialogical texts to be specified in the form of 'speaker:text' formats. 
		It takes input in both text and xIAF formats (where the input is specified in the text field of xIAF) to return xIAF as an output. 
		Please note that the argument type (dialog vs monolog) can be specified in xIAF if the input is in xIAF format; otherwise, it is considered a monological argument. 
		The component can be used as a starting point for creating an argument mining pipeline that takes monological or dialogical text specified as regular text or xIAF."""
		return info
	
if __name__ == "__main__":
    app.run(host="0.0.0.0", port=int("5006"), debug=False)	  
